(ns com.aurellem.gb-driver
  (:import com.aurellem.gb.Gb)
  (:import java.io.File)
  (:import (java.nio IntBuffer ByteOrder)))

(Gb/loadVBA)

(def yellow-rom-image
  (File. "/home/r/proj/pokemon-escape/roms/yellow.gbc"))

(defn vba-init []
  (future
    (Gb/startEmulator (.getCanonicalPath yellow-rom-image))))

(defn cpu-data [size arr-fn]
  (let [store (int-array size)]
    (fn [] 
      (arr-fn store)
      store)))

(def ram
  (cpu-data (Gb/getRAMSize) #(Gb/getRAM %)))

(def rom 
  (cpu-data (Gb/getROMSize) #(Gb/getROM %)))

(def working-ram 
  (cpu-data Gb/WRAM_SIZE #(Gb/getWRAM %)))

(def video-ram 
  (cpu-data Gb/VRAM_SIZE #(Gb/getVRAM %)))

(def registers
  (cpu-data Gb/NUM_REGISTERS #(Gb/getRegisters %)))


(def key-mask
  {:a         0x0001
   :b         0x0002

   :l         0x0020
   :r         0x0010
   :u         0x0040
   :d         0x0080

   :0         0x0000
   
   :select    0x0004
   :start     0x0008
   :restart   0x0800})

(defn keycode [& keys]
  (reduce bit-or (map key-mask keys)))

(defn step
  ([] (Gb/step))
  ([& keys] (Gb/step (apply keycode keys))))

   
  

